﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework.Graphics;
    
namespace Application1
{
    public partial class MainForm : Form
    {
        // Объявим поле графического устройства для видимости в методах
        GraphicsDevice device;
        // Объявим объект настроек графического устройства 
        PresentationParameters presentParams;
    
        public MainForm()
        {
            InitializeComponent();
    
            // Подпишемся на событие Load формы 
            this.Load += new EventHandler(MainForm_Load);
    
            // Попишемся на событие FormClosed формы
            this.FormClosed += new FormClosedEventHandler(MainForm_FormClosed);
        }
    
        void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            //  Удаляем (освобождаем) устройство
            device.Dispose();
            // На всякий случай присваиваем ссылке на устройство значение null 
            device = null;        
        }
    
        void MainForm_Load(object sender, EventArgs e)
        {
            // Создаем объект представления для настройки графического устройства
            //PresentationParameters presentParams = new PresentationParameters();
            presentParams = new PresentationParameters();
            // Настраиваем объект представления через его свойства
            presentParams.IsFullScreen = false; // Включаем оконный режим
            presentParams.BackBufferCount = 1;  // Включаем задний буфер 
                                                // для двойной буферизации 
            // Переключение переднего и заднего буферов 
            // должно осуществляться с максимальной эффективностью
            presentParams.SwapEffect = SwapEffect.Discard;
            // Устанавливаем размеры заднего буфера по клиентской области окна формы
            presentParams.BackBufferWidth = this.ClientSize.Width;
            presentParams.BackBufferHeight = this.ClientSize.Height;
    
            // Создадим графическое устройство с заданными настройками
            device = new GraphicsDevice(GraphicsAdapter.DefaultAdapter, DeviceType.Hardware,
                this.Handle, presentParams);
    
            // Перерисовывать форму при изменении размеров 
            this.SetStyle(ControlStyles.ResizeRedraw, true);
            // Запретить очистку экрана средствами GDI+ 
            this.SetStyle(ControlStyles.Opaque, true);
    
            // Вычисляем размер окна при размерах клиентской области 1x1 пикселов.
            // Полученное значение присваиваем свойству MinimumSize 
            this.MinimumSize = this.SizeFromClientSize(new Size(1, 1));
        }
    
        protected override void OnResize(EventArgs e)
        {
            // Устанавливаем размеры заднего буфера по клиентской области окна формы
            presentParams.BackBufferWidth = this.ClientSize.Width;
            presentParams.BackBufferHeight = this.ClientSize.Height;
            // Сбрасываем устройство и применяем к нему новые параметры  
            // Если окно не минимизировано
            if (this.WindowState != FormWindowState.Minimized)  
                device.Reset(presentParams);
    
            base.OnResize(e);
        }
    
        bool closing = false;
        protected override void OnPaint(PaintEventArgs e)
        {
            if (closing) return;
    
            try     // Попытка
            {
                // Очищаем экран белым цветом
                device.Clear(Microsoft.Xna.Framework.Graphics.Color.WhiteSmoke);
    
                // Создаeм массив областей закраски, соответствующих коричневым клеткам
                Microsoft.Xna.Framework.Rectangle[] rects = new Microsoft.Xna.Framework.Rectangle[32];
                int k = 0; // Счетчик элементов массива 
                // Перебираем коричневые клетки шахматной доски 
                for (int j = 0; j < 8; j++) // Строки шахматной доски
                    for (int i = j % 2; i < 8; i += 2)  // Столбцы шахматной доски
                    {
                        // Заносим в массив параметры рисования очередной клетки
                        rects[k] = new Microsoft.Xna.Framework.Rectangle(
                            i * this.ClientSize.Width / 8,  // Отступ по горизонтали
                            j * this.ClientSize.Height / 8, // Отступ по вертикали
                            this.ClientSize.Width / 8,      // Ширина клетки
                            this.ClientSize.Height / 8);    // Высота клетки
    
                        k++;    // Увеличиваем счетчик
                    }
    
                // Закрашиваем все области из массива rects коричневым цветом
                device.Clear(ClearOptions.Target, Microsoft.Xna.Framework.
                    Graphics.Color.Brown, 0.0f, 0, rects);
    
                // Копируем задний буфер на экран
                device.Present();
    
                base.OnPaint(e);
            }
            // Откаты
            catch (DeviceNotResetException) // Устройство можно восстановить
            {
                device.Reset(presentParams);
                this.Invalidate();
            }
            catch (DeviceLostException)     // Устройство нельзя восстановить
            {
                closing = true;     // Больше не выполнять OnPaint()
                string title = "Сбой графического устройства";
                string message = "Работа программы будет завершена.\n"
                               + "Закройте все ненужные программы\n"
                               + "и повторите запуск этого приложения";
                MessageBox.Show(message, title, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                Application.Idle += new EventHandler(Application_Idle);
                return;
            }
            catch (Exception ext)           // Все другие исключения
            {
                closing = true;
                MessageBox.Show(ext.Message);
                Application.Idle += Application_Idle1;
            }
        }
    
        void Application_Idle(object sender, EventArgs e)
        {
            this.Close();   // Закрывает окно, у нас оно же и главное окно приложения
        }
    
        void Application_Idle1(object sender, EventArgs e)
        {
            Application.Exit(); // Завершает работу приложения (приведено для разнообразия)
        }
    }
}
